local class = require("silo.lib.class")
local trace = require("silo.lib.trace")

local EventBus = require("silo.event_bus")
local Config = require("silo.lib.config")

local UNINITIALIZED_MODULE = { __uninit = true }
local Context = class("silo.Context")

function Context:new()
  self.bus = EventBus()
  self.config = Config("/.config/silo.lua")
  self.modules = {}
  self.tasks = {}
end

function Context:require(module)
  if type(module) == "string" then
    module = require(module)
  end

  local instance = self.modules[module]
  if instance == UNINITIALIZED_MODULE then
    error("Detected cyclic dependency: " .. tostring(module), 2)
    return nil
  end

  if instance == nil then
    self.modules[module] = UNINITIALIZED_MODULE
    instance = module(self)
    self.modules[module] = instance
  end

  return instance
end

function Context:addTask(task)
  if type(task) ~= "function" then error("bad argument #1 (function expected, got " .. type(task) .. ")") end
  table.insert(self.tasks, trace.into(task))
end

function Context:run()
  local ok, res = pcall(parallel.waitForAll, table.unpack(self.tasks))

  if not ok then
    local current = term.current()
    if current.endPrivateMode then current.endPrivateMode() end
    error(res, 0)
  end

  return res
end

return Context
